@@ -25,7 +25,7 @@
 import os
 import gi
 import logging
-import locale
+import gettext
 import builtins
 
 from . import config
@@ -37,19 +37,15 @@ gi.require_version("GtkSource", "5")
 gi.require_version("WebKit", "6.0")
 gi.require_version('Adw', '1')
 
-# Ensure _() builtin
-if "_" not in builtins.__dict__:
-    _ = locale.gettext
-
-if "N_" not in builtins.__dict__:
-
-    def N_(s, p, n):
-        return _(p) if n > 1 else _(s)
-
-
-# noqa: E402,E401
+# flake8: noqa: E402,F401
 from gi.repository import Gio, Gdk, Gtk
 
+# Ensure _() builtin
+if "_" not in builtins.__dict__:
+    _ = gettext.gettext
+
+if "ngettext" not in builtins.__dict__:
+    ngettext = gettext.ngettext
 
 resource = Gio.Resource.load(os.path.join(config.pkgdatadir, "cambalache.gresource"))
 resource._register()
@@ -75,7 +71,6 @@ def getLogger(name):
 
     return logger
 
-
 from .cmb_base_objects import CmbBaseObject
 from .cmb_base_file_monitor import CmbBaseFileMonitor
 from .cmb_css import CmbCSS
@@ -109,3 +104,4 @@ from .cmb_accessible_editor import CmbAccessibleEditor
 from .cmb_type_chooser import CmbTypeChooser
 from .cmb_type_chooser_widget import CmbTypeChooserWidget
 from .cmb_type_chooser_popover import CmbTypeChooserPopover
+from .cmb_project_settings import CmbProjectSettings
