@@ -23,31 +23,30 @@
 # SPDX-License-Identifier: LGPL-2.1-only
 #
 
-import gi
 import os
 import locale
 import tempfile
 
-gi.require_version('CambalachePrivate', '4.0')
 from gi.repository import GLib, GObject, Gio, Gdk, Gtk, Pango, Adw, GtkSource, CambalachePrivate
 from .cmb_tutor import CmbTutor, CmbTutorState
 from . import cmb_tutorial
 
 from cambalache import (
-    CmbProject,
     CmbBaseFileMonitor,
-    CmbUI,
     CmbCSS,
-    CmbObject,
     CmbGResource,
     CmbGResourceEditor,
+    CmbObject,
+    CmbProject,
+    CmbProjectSettings,
     CmbTypeChooserPopover,
-    getLogger,
-    notification_center,
-    config,
-    utils,
+    CmbUI,
     _,
-    N_
+    config,
+    getLogger,
+    ngettext,
+    notification_center,
+    utils,
 )
 
 from cambalache.cmb_blueprint import CmbBlueprintError
@@ -167,8 +166,6 @@ class CmbWindow(Adw.ApplicationWindow):
 
         self.__recent_manager = self.__get_recent_manager()
 
-        self.editor_stack.set_size_request(420, -1)
-
         self.actions = {}
 
         for action in [
@@ -207,6 +204,7 @@ class CmbWindow(Adw.ApplicationWindow):
             "save",
             "save_as",
             "select_project_location",
+            "settings",
             "show_workspace",
             "undo",
             "workspace_restart",
@@ -312,6 +310,8 @@ class CmbWindow(Adw.ApplicationWindow):
         for prop in settings:
             self.settings.bind(prop, self, prop.replace("-", "_"), Gio.SettingsBindFlags.DEFAULT)
 
+        # Force minimum fize
+        self.set_default_size(320, 240)
         self.__load_window_state()
         self.__update_actions()
 
@@ -677,15 +677,16 @@ class CmbWindow(Adw.ApplicationWindow):
         has_project = self.__is_project_visible()
 
         for action in [
-            "save_as",
-            "add_ui",
             "add_css",
             "add_gresource",
+            "add_ui",
+            "close",
+            "debug",
             "delete",
             "import",
             "import_directory",
-            "close",
-            "debug"
+            "save_as",
+            "settings",
         ]:
             self.actions[action].set_enabled(has_project)
 
@@ -1213,6 +1214,8 @@ class CmbWindow(Adw.ApplicationWindow):
             filters=import_filters,
             accept_label=_("Import")
         )
+        if self.project.dirname:
+            dialog.set_initial_folder(Gio.File.new_for_path(self.project.dirname))
         dialog.open_multiple(self, None, dialog_callback)
 
     def _on_import_directory_activate(self, action, data):
@@ -1244,17 +1247,17 @@ class CmbWindow(Adw.ApplicationWindow):
         def dialog_callback(dialog, res):
             main_loop = GLib.MainContext.default()
 
-            progress, progressbar = progress_dialog_new()
-
-            def pulse():
-                progressbar.pulse()
-                while main_loop.pending():
-                    main_loop.iteration(False)
-
             try:
                 dir = dialog.select_folder_finish(res)
                 dirpath = dir.get_path()
 
+                progress, progressbar = progress_dialog_new()
+
+                def pulse():
+                    progressbar.pulse()
+                    while main_loop.pending():
+                        main_loop.iteration(False)
+
                 files = self.project._list_supported_files(dirpath, pulse)
                 n_files = len(files)
 
@@ -1293,7 +1296,8 @@ class CmbWindow(Adw.ApplicationWindow):
                 logger.warning(f"Error {e}")
 
         dialog = self.__file_open_dialog_new(_("Choose directory to import"), accept_label=_("Import directory"))
-        dialog.set_initial_folder(Gio.File.new_for_path(self.project.dirname))
+        if self.project.dirname:
+            dialog.set_initial_folder(Gio.File.new_for_path(self.project.dirname))
         dialog.select_folder(self, None, dialog_callback)
 
     def _on_add_gresource_activate(self, action, data):
@@ -1311,11 +1315,7 @@ class CmbWindow(Adw.ApplicationWindow):
         except CmbBlueprintError as e:
             self.present_message_to_user(
                 _("Error saving project"),
-                secondary_text=N_(
-                    "blueprintcompiler encounter the following error:",
-                    "blueprintcompiler encounter the following errors:",
-                    len(e.errors)
-                ),
+                secondary_text=ngettext("blueprintcompiler encounter the following error:", "blueprintcompiler encounter the following errors:", len(e.errors)),
                 details=[str(e)]
             )
         finally:
@@ -1398,6 +1398,13 @@ class CmbWindow(Adw.ApplicationWindow):
         else:
             close_project()
 
+    def _on_settings_activate(self, action, data):
+        if self.project is None:
+            return
+
+        settings = CmbProjectSettings(project=self.project)
+        settings.present(self)
+
     def _on_debug_activate(self, action, data):
         if self.project.filename:
             filename = self.project.filename + ".db"
@@ -1622,8 +1629,9 @@ class CmbWindow(Adw.ApplicationWindow):
         elif state & self.FULLSCREEN:
             self.fullscreen()
         else:
-            size = self.window_settings.get_value("size").unpack()
-            self.set_default_size(*size)
+            w, h = self.window_settings.get_value("size").unpack()
+            if w and h:
+                self.set_default_size(w, h)
 
     def __save_window_state(self):
         fullscreen = self.props.fullscreened
